@@ -8,6 +8,13 @@
 <body>
 <?php
 
+if (!extension_loaded('ice')) {
+	exit('Error: This example script needs the PHP extension for Ice to be loaded.');
+}
+if ((!function_exists('Ice_intVersion') || Ice_intVersion() < 30400) || !function_exists('Ice_loadProfile')) {
+	exit('Error: This example script is compatible only for php-ice versions prior to 3.4.');
+}
+
 Ice_loadProfile();
 
 if ($argc > 0) {
@@ -52,38 +59,42 @@ try {
   if (! is_null($_REQUEST['newserver'])) {
     $meta->newServer();
   } else if (! is_null($_REQUEST['delserver'])) {
-    $meta->getServer($_REQUEST['delserver'] + 0)->delete();
+    $meta->getServer((int)$_REQUEST['delserver'])->delete();
   } else if (! is_null($_REQUEST['stop'])) {
-    $meta->getServer($_REQUEST['stop'] + 0)->stop();
+    $meta->getServer((int) $_REQUEST['stop'])->stop();
   } else if (! is_null($_REQUEST['start'])) {
-    $meta->getServer($_REQUEST['start'] + 0)->start();
+    $meta->getServer((int) $_REQUEST['start'])->start();
   } else if (! is_null($_REQUEST['action'])) {
-    $server = $meta->getServer($_REQUEST['action'] + 0);
-    if (! is_null($_REQUEST['kick'])) {
-      $server->kickPlayer($_REQUEST['kick'] + 0, "Mushroom");
+    $server = $meta->getServer((int) $_REQUEST['action']);
+    if (!is_null($_REQUEST['kick'])) {
+      $server->kickPlayer((int) $_REQUEST['kick'], "Mushroom");
     }
   } else if (! is_null($_REQUEST['uedit'])) {
-    $server = $meta->getServer($_REQUEST['uedit'] + 0);
+    $server = $meta->getServer((int) $_REQUEST['uedit']);
     if (isset($_REQUEST['newplayer'])) {
-      $_REQUEST['uid'] = $server->registerPlayer($_REQUEST['newplayer']);
+      $reg = array();
+      $reg[Murmur_UserInfo::UserName] = $_REQUEST['newplayer'];
+	  if ($_REQUEST['password']) $reg[Murmur_UserInfo::UserPassword] = $_REQUEST['password']; // This line is added for password at user creation
+      $_REQUEST['uid'] = $server->registerUser($reg);
     }
     if (! is_null($_REQUEST['deleteplayer'])) {
-      $server->unregisterPlayer($_REQUEST['deleteplayer'] + 0);
+      $server->unregisterUser((int) $_REQUEST['deleteplayer']);
     }
     if (! is_null($_REQUEST['uid'])) {
-      $user = $server->getRegistration($_REQUEST['uid'] + 0);
+      $uid = (int) $_REQUEST['uid'];
+      $user = $server->getRegistration($uid);
       if (! is_null($_REQUEST['set'])) {
-        $user->email = $_REQUEST['email'];
-        $user->pw = $_REQUEST['pw'];
-        $server->updateRegistration($user);
+        $user[Murmur_UserInfo::UserEmail] = $_REQUEST['email'];
+        $user[Murmur_UserInfo::UserPassword] = $_REQUEST['pw'];
+        $server->updateRegistration($uid,$user);
       } else {
         echo "<form method=\"post\" action=\"".$_SERVER['PHP_SELF']."\">\n";
         echo "<p>\n";
-        echo "<b>Name:</b> $user->name<br />\n";
+        echo "<b>Name:</b> ".$user[Murmur_UserInfo::UserName]."<br />\n";
         echo "<input type=\"hidden\" name=\"set\" value=\"1\" />\n";
         echo "<input type=\"hidden\" name=\"uedit\" value=\"".$server->id()."\" />\n";
-        echo "<input type=\"hidden\" name=\"uid\" value=\"$user->playerid\" />\n";
-        echo "<b>Email:</b> <input type=\"text\" name=\"email\" size=\"30\" maxlength=\"128\" value=\"".htmlspecialchars($user->email)."\" /><br />\n";
+        echo "<input type=\"hidden\" name=\"uid\" value=\"$uid\" />\n";
+        echo "<b>Email:</b> <input type=\"text\" name=\"email\" size=\"30\" maxlength=\"128\" value=\"".htmlspecialchars($user[Murmur_UserInfo::UserEmail])."\" /><br />\n";
         echo "<b>New password:</b> <input type=\"password\" name=\"pw\" size=\"30\" maxlength=\"128\" /><br />\n";
         echo "<input type=\"submit\" />\n";
         echo "</p>\n";
@@ -96,17 +107,19 @@ try {
     echo "<input type=\"hidden\" name=\"uedit\" value=\"".$server->id()."\" />\n";
     echo "<b>New User:</b>";
     echo "<input type=\"text\" name=\"newplayer\" size=\"30\" maxlength=\"60\" />";
+	echo "<b>User Password:</b>";
+    echo "<input type=\"text\" name=\"password\" size=\"30\" maxlength=\"60\" />";
     echo "<input type=\"submit\" />\n";
     echo "</p>\n";
     echo "</form>\n";
     echo "<table>\n";
     echo "<tr><th>UserName</th><th>Email</th><th></th></tr>\n";
-    $users = $server->getRegisteredPlayers("");
-    usort($users, "ucmp");
-    foreach($users as $u) {
-      echo "<tr><td>$u->name</td><td>".$u->email."</td><td>";
-      echo "<a href=\"?uedit=".$server->id()."&amp;uid=".$u->playerid."\">[Edit]</a> ";
-      echo "<a href=\"?uedit=".$server->id()."&amp;deleteplayer=".$u->playerid."\">[Unregister]</a> ";
+    $users = $server->getRegisteredUsers("");
+    foreach($users as $id=>$name) {
+      $u = $server->getRegistration($id);
+      echo "<tr><td>$name $id</td><td>".$u[Murmur_UserInfo::UserEmail]."</td><td>";
+      echo "<a href=\"?uedit=".$server->id()."&amp;uid=".$id."\">[Edit]</a> ";
+      echo "<a href=\"?uedit=".$server->id()."&amp;deleteplayer=".$id."\">[Unregister]</a> ";
       echo "</td></tr>\n";
     }
     echo "</table>\n";
@@ -174,7 +187,7 @@ try {
       echo "<table><tr><th>Name</th><th>Channel</th><th>Actions</th></tr>\n";
 
       $channels = $s->getChannels();
-      $players = $s->getPlayers();
+      $players = $s->getUsers();
 
       foreach($players as $pid => $state) {
         $chan = $channels[$state->channel];
