@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -28,17 +28,26 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#include "mumble_pch.hpp"
+
 #include "AudioInput.h"
+
 #include "AudioOutput.h"
+#include "CELTCodec.h"
 #include "ServerHandler.h"
 #include "MainWindow.h"
 #include "User.h"
+#include "PacketDataStream.h"
 #include "Plugins.h"
 #include "Message.h"
 #include "Global.h"
 #include "NetworkConfig.h"
 #include "VoiceRecorder.h"
 
+#ifdef USE_OPUS
+#include "opus.h"
+#endif
+
 // Remember that we cannot use static member classes that are not pointers, as the constructor
 // for AudioInputRegistrar() might be called before they are initialized, as the constructor
 // is called from global initialization.
@@ -87,56 +96,31 @@ bool AudioInputRegistrar::canExclusive() const {
 	return false;
 }
 
-AudioInput::AudioInput() {
+AudioInput::AudioInput() : opusBuffer(g.s.iFramesPerPacket * (SAMPLE_RATE / 100)) {
 	adjustBandwidth(g.iMaxBandwidth, iAudioQuality, iAudioFrames);
 
 	g.iAudioBandwidth = getNetworkBandwidth(iAudioQuality, iAudioFrames);
 
-	if (preferCELT(iAudioQuality, iAudioFrames))
-		umtType = MessageHandler::UDPVoiceCELTAlpha;
-	else
-		umtType = MessageHandler::UDPVoiceSpeex;
+	umtType = MessageHandler::UDPVoiceCELTAlpha;
 
 	cCodec = NULL;
 	ceEncoder = NULL;
 
-	if (umtType != MessageHandler::UDPVoiceSpeex) {
-		iSampleRate = SAMPLE_RATE;
-		iFrameSize = SAMPLE_RATE / 100;
+	iSampleRate = SAMPLE_RATE;
+	iFrameSize = SAMPLE_RATE / 100;
 
-		esSpeex = NULL;
-		qWarning("AudioInput: %d bits/s, %d hz, %d sample CELT", iAudioQuality, iSampleRate, iFrameSize);
-	} else {
-		iAudioFrames /= 2;
+#ifdef USE_OPUS
+	opusState = opus_encoder_create(SAMPLE_RATE, 1, OPUS_APPLICATION_VOIP, NULL);
+	opus_encoder_ctl(opusState, OPUS_SET_VBR(0)); // CBR
+#endif
 
-		speex_bits_init(&sbBits);
-		speex_bits_reset(&sbBits);
-		esSpeex = speex_encoder_init(speex_lib_get_mode(SPEEX_MODEID_UWB));
-		speex_encoder_ctl(esSpeex,SPEEX_GET_FRAME_SIZE,&iFrameSize);
-		speex_encoder_ctl(esSpeex,SPEEX_GET_SAMPLING_RATE,&iSampleRate);
-
-		int iArg=1;
-		speex_encoder_ctl(esSpeex,SPEEX_SET_VBR, &iArg);
-
-		iArg = 0;
-		speex_encoder_ctl(esSpeex,SPEEX_SET_VAD, &iArg);
-		speex_encoder_ctl(esSpeex,SPEEX_SET_DTX, &iArg);
-
-		float fArg=8.0;
-		speex_encoder_ctl(esSpeex,SPEEX_SET_VBR_QUALITY, &fArg);
-
-		iArg = iAudioQuality;
-		speex_encoder_ctl(esSpeex, SPEEX_SET_VBR_MAX_BITRATE, &iArg);
-
-		iArg = 5;
-		speex_encoder_ctl(esSpeex,SPEEX_SET_COMPLEXITY, &iArg);
-		qWarning("AudioInput: %d bits/s, %d hz, %d sample Speex-UWB", iAudioQuality, iSampleRate, iFrameSize);
-	}
+	qWarning("AudioInput: %d bits/s, %d hz, %d sample", iAudioQuality, iSampleRate, iFrameSize);
 	iEchoFreq = iMicFreq = iSampleRate;
 
 	iFrameCounter = 0;
 	iSilentFrames = 0;
 	iHoldFrames = 0;
+	iBufferedFrames = 0;
 
 	bResetProcessor = true;
 
@@ -172,17 +156,20 @@ AudioInput::AudioInput() {
 	bRunning = true;
 
 	connect(this, SIGNAL(doDeaf()), g.mw->qaAudioDeaf, SLOT(trigger()), Qt::QueuedConnection);
+	connect(this, SIGNAL(doMute()), g.mw->qaAudioMute, SLOT(trigger()), Qt::QueuedConnection);
 }
 
 AudioInput::~AudioInput() {
 	bRunning = false;
 	wait();
 
+#ifdef USE_OPUS
+	if (opusState)
+		opus_encoder_destroy(opusState);
+#endif
+
 	if (ceEncoder) {
 		cCodec->celt_encoder_destroy(ceEncoder);
-	} else if (esSpeex) {
-		speex_bits_destroy(&sbBits);
-		speex_encoder_destroy(esSpeex);
 	}
 
 	foreach(short *buf, qlEchoFrames)
@@ -368,13 +355,16 @@ void AudioInput::initializeMixer() {
 
 void AudioInput::addMic(const void *data, unsigned int nsamp) {
 	while (nsamp > 0) {
-		unsigned int left = qMin(nsamp, iMicLength - iMicFilled);
+		// Make sure we don't overrun the frame buffer
+		const unsigned int left = qMin(nsamp, iMicLength - iMicFilled);
 
+		// Append mix into pfMicInput frame buffer (converts 16bit pcm->float if necessary)
 		imfMic(pfMicInput + iMicFilled, data, left, iMicChannels);
 
 		iMicFilled += left;
 		nsamp -= left;
 
+		// If new samples are left offset data pointer to point at the first one for next iteration
 		if (nsamp > 0) {
 			if (eMicFormat == SampleFloat)
 				data = reinterpret_cast<const float *>(data) + left * iMicChannels;
@@ -383,18 +373,24 @@ void AudioInput::addMic(const void *data, unsigned int nsamp) {
 		}
 
 		if (iMicFilled == iMicLength) {
+			// Frame complete
 			iMicFilled = 0;
 
+			// If needed resample frame
 			float *ptr = srsMic ? pfOutput : pfMicInput;
+
 			if (srsMic) {
 				spx_uint32_t inlen = iMicLength;
 				spx_uint32_t outlen = iFrameSize;
 				speex_resampler_process_float(srsMic, 0, pfMicInput, &inlen, pfOutput, &outlen);
 			}
-			const float mul = 32768.f;
-			for (int j=0;j<iFrameSize;++j)
-				psMic[j] = static_cast<short>(ptr[j] * mul);
 
+			// Convert float to 16bit PCM
+			const float mul = 32768.f;
+			for (int j = 0; j < iFrameSize; ++j)
+				psMic[j] = static_cast<short>(qBound(-32768.f, (ptr[j] * mul), 32767.f));
+
+			// If we have echo chancellation enabled...
 			if (iEchoChannels > 0) {
 				short *echo = NULL;
 
@@ -405,6 +401,7 @@ void AudioInput::addMic(const void *data, unsigned int nsamp) {
 						iJitterSeq = 0;
 						iMinBuffered = 1000;
 					} else {
+						// Compensate for drift between the microphone and the echo source
 						iMinBuffered = qMin(iMinBuffered, qlEchoFrames.count());
 
 						if ((iJitterSeq > 100) && (iMinBuffered > 1)) {
@@ -417,10 +414,13 @@ void AudioInput::addMic(const void *data, unsigned int nsamp) {
 				}
 
 				if (echo) {
+					// We have echo data for the current frame, remember that
 					delete [] psSpeaker;
 					psSpeaker = echo;
 				}
 			}
+
+			// Encode and send frame
 			encodeAudioFrame();
 		}
 	}
@@ -428,23 +428,30 @@ void AudioInput::addMic(const void *data, unsigned int nsamp) {
 
 void AudioInput::addEcho(const void *data, unsigned int nsamp) {
 	while (nsamp > 0) {
-		unsigned int left = qMin(nsamp, iEchoLength - iEchoFilled);
+		// Make sure we don't overrun the echo frame buffer
+		const unsigned int left = qMin(nsamp, iEchoLength - iEchoFilled);
 
 		if (bEchoMulti) {
 			const unsigned int samples = left * iEchoChannels;
-			if (eEchoFormat == SampleFloat)
+
+			if (eEchoFormat == SampleFloat) {
 				for (unsigned int i=0;i<samples;++i)
 					pfEchoInput[i] = reinterpret_cast<const float *>(data)[i];
-			else
+			}
+			else {
+				// 16bit PCM -> float
 				for (unsigned int i=0;i<samples;++i)
 					pfEchoInput[i] = static_cast<float>(reinterpret_cast<const short *>(data)[i]) * (1.0f / 32768.f);
+			}
 		} else {
+			// Mix echo channels (converts 16bit PCM -> float if needed)
 			imfEcho(pfEchoInput + iEchoFilled, data, left, iEchoChannels);
 		}
 
 		iEchoFilled += left;
 		nsamp -= left;
 
+		// If new samples are left offset data pointer to point at the first one for next iteration
 		if (nsamp > 0) {
 			if (eEchoFormat == SampleFloat)
 				data = reinterpret_cast<const float *>(data) + left * iEchoChannels;
@@ -453,9 +460,13 @@ void AudioInput::addEcho(const void *data, unsigned int nsamp) {
 		}
 
 		if (iEchoFilled == iEchoLength) {
+			//Frame complete
+
 			iEchoFilled = 0;
 
+			// Resample if necessary
 			float *ptr = srsEcho ? pfOutput : pfEchoInput;
+
 			if (srsEcho) {
 				spx_uint32_t inlen = iEchoLength;
 				spx_uint32_t outlen = iFrameSize;
@@ -464,22 +475,20 @@ void AudioInput::addEcho(const void *data, unsigned int nsamp) {
 
 			short *outbuff = new short[iEchoFrameSize];
 
+			// float -> 16bit PCM
 			const float mul = 32768.f;
 			for (unsigned int j=0;j<iEchoFrameSize;++j)
 				outbuff[j] = static_cast<short>(ptr[j] * mul);
 
-			iJitterSeq = qMin(iJitterSeq+1,10000U);
-
+			// Push frame into the echo chancellers jitter buffer
 			QMutexLocker l(&qmEcho);
+
+			iJitterSeq = qMin(iJitterSeq + 1,10000U);
 			qlEchoFrames.append(outbuff);
 		}
 	}
 }
 
-bool AudioInput::preferCELT(int bitrate, int frames) {
-	return ((bitrate >= 32000) || (frames == 1));
-}
-
 void AudioInput::adjustBandwidth(int bitspersec, int &bitrate, int &frames) {
 	frames = g.s.iFramesPerPacket;
 	bitrate = g.s.iQuality;
@@ -517,11 +526,11 @@ void AudioInput::setMaxBandwidth(int bitspersec) {
 
 	if (bitspersec != -1) {
 		if ((bitrate != g.s.iQuality) || (frames != g.s.iFramesPerPacket))
-			g.mw->msgBox(tr("Server maximum network bandwidth is only %1 kbit/s. Audio quality auto-adjusted to %2 kbit/s (%3ms)").arg(bitspersec / 1000).arg(bitrate / 1000).arg(frames*10));
+			g.mw->msgBox(tr("Server maximum network bandwidth is only %1 kbit/s. Audio quality auto-adjusted to %2 kbit/s (%3 ms)").arg(bitspersec / 1000).arg(bitrate / 1000).arg(frames*10));
 	}
 
 	AudioInputPtr ai = g.ai;
-	if (ai && (preferCELT(bitrate, frames) == (ai->umtType != MessageHandler::UDPVoiceSpeex))) {
+	if (ai) {
 		g.iAudioBandwidth = getNetworkBandwidth(bitrate, frames);
 		ai->iAudioQuality = bitrate;
 		ai->iAudioFrames = frames;
@@ -542,57 +551,158 @@ int AudioInput::getNetworkBandwidth(int bitrate, int frames) {
 	return bw;
 }
 
+void AudioInput::resetAudioProcessor() {
+	if (!bResetProcessor)
+		return;
 
-/*
- * FIXME: Experimental zero-latency callback-based preprocessing.
+	int iArg;
 
-extern "C" {
-	int speex_preprocess_run_cb(SpeexPreprocessState *st, float *ft);
-	SpeexPreprocessState *speex_preprocess_callback_init(int frame_size, int sampling_rate);
-};
+	if (sppPreprocess)
+		speex_preprocess_state_destroy(sppPreprocess);
+	if (sesEcho)
+		speex_echo_state_destroy(sesEcho);
 
-celt_int32_t celtBack(CELTEncoder *enc, void *rawdata, celt_int32_t format, celt_int32_t bits, celt_int32_t num, void *data) {
-	qWarning() << "CB" << enc << rawdata << format << bits << num << data;
+	sppPreprocess = speex_preprocess_state_init(iFrameSize, iSampleRate);
 
-	static SpeexPreprocessState *st = NULL;
-	if (! st) {
-		int samp = 480;
-		int freq = 48000;
+	iArg = 1;
+	speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_VAD, &iArg);
+	speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_AGC, &iArg);
+	speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_DENOISE, &iArg);
+	speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_DEREVERB, &iArg);
 
-		celt_encoder_ctl(enc, CELT_GET_SAMPLE_RATE, &freq);
-		celt_encoder_ctl(enc, CELT_GET_FRAME_SIZE, &samp);
+	iArg = 30000;
+	speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_AGC_TARGET, &iArg);
 
-		qWarning() << "Init" << samp << freq;
-		st= speex_preprocess_callback_init(samp, freq);
+	float v = 30000.0f / static_cast<float>(g.s.iMinLoudness);
+	iArg = iroundf(floorf(20.0f * log10f(v)));
+	speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_AGC_MAX_GAIN, &iArg);
 
-		int iArg;
+	iArg = -60;
+	speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_AGC_DECREMENT, &iArg);
 
-		iArg = 1;
-		speex_preprocess_ctl(st, SPEEX_PREPROCESS_SET_VAD, &iArg);
-		speex_preprocess_ctl(st, SPEEX_PREPROCESS_SET_AGC, &iArg);
-		speex_preprocess_ctl(st, SPEEX_PREPROCESS_SET_DENOISE, &iArg);
-		speex_preprocess_ctl(st, SPEEX_PREPROCESS_SET_DEREVERB, &iArg);
+	iArg = g.s.iNoiseSuppress;
+	speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_NOISE_SUPPRESS, &iArg);
 
-		iArg = 30000;
-		speex_preprocess_ctl(st, SPEEX_PREPROCESS_SET_AGC_TARGET, &iArg);
+	if (iEchoChannels > 0) {
+		sesEcho = speex_echo_state_init_mc(iFrameSize, iFrameSize * 10, 1, bEchoMulti ? iEchoChannels : 1);
+		iArg = iSampleRate;
+		speex_echo_ctl(sesEcho, SPEEX_ECHO_SET_SAMPLING_RATE, &iArg);
+		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_ECHO_STATE, sesEcho);
 
-		iArg = 30000;
-		speex_preprocess_ctl(st, SPEEX_PREPROCESS_SET_AGC_TARGET, &iArg);
-
-		float v = 30000.0f / static_cast<float>(g.s.iMinLoudness);
-		iArg = iroundf(floorf(20.0f * log10f(v)));
-		speex_preprocess_ctl(st, SPEEX_PREPROCESS_SET_AGC_MAX_GAIN, &iArg);
-
-		iArg = g.s.iNoiseSuppress;
-		speex_preprocess_ctl(st, SPEEX_PREPROCESS_SET_NOISE_SUPPRESS, &iArg);
+		qWarning("AudioInput: ECHO CANCELLER ACTIVE");
+	} else {
+		sesEcho = NULL;
 	}
-	qWarning() << "Call";
-//	int res = 1;
-	int res = speex_preprocess_run_cb(st, (float *) rawdata);
-	qWarning() << "Result" << res;
-	return 1;
+
+	bResetProcessor = false;
+}
+
+bool AudioInput::selectCodec() {
+	bool useOpus = false;
+
+	// Currently talking, use previous Opus status.
+	if (bPreviousVoice) {
+		useOpus = (umtType == MessageHandler::UDPVoiceOpus);
+	} else {
+#ifdef USE_OPUS
+		if (g.bOpus || (g.s.lmLoopMode == Settings::Local)) {
+			useOpus = true;
+		}
+#endif
+	}
+
+	if (!useOpus) {
+		CELTCodec *switchto = NULL;
+		if ((!g.uiSession || (g.s.lmLoopMode == Settings::Local)) && (!g.qmCodecs.isEmpty())) {
+			// Use latest for local loopback
+			QMap<int, CELTCodec *>::const_iterator i = g.qmCodecs.constEnd();
+			--i;
+			switchto = i.value();
+		} else {
+			// Currently talking, don't switch unless you must.
+			if (cCodec && bPreviousVoice) {
+				int v = cCodec->bitstreamVersion();
+				if ((v == g.iCodecAlpha) || (v == g.iCodecBeta))
+					switchto = cCodec;
+			}
+		}
+		if (!switchto) {
+			switchto = g.qmCodecs.value(g.bPreferAlpha ? g.iCodecAlpha : g.iCodecBeta);
+			if (!switchto)
+				switchto = g.qmCodecs.value(g.bPreferAlpha ? g.iCodecBeta : g.iCodecAlpha);
+		}
+		if (switchto != cCodec) {
+			if (cCodec && ceEncoder) {
+				cCodec->celt_encoder_destroy(ceEncoder);
+				ceEncoder = NULL;
+			}
+			cCodec = switchto;
+			if (cCodec)
+				ceEncoder = cCodec->encoderCreate();
+		}
+
+		if (!cCodec)
+			return false;
+	}
+
+	MessageHandler::UDPMessageType previousType = umtType;
+	if (useOpus) {
+		umtType = MessageHandler::UDPVoiceOpus;
+	} else {
+		if (!g.uiSession) {
+			umtType = MessageHandler::UDPVoiceCELTAlpha;
+		} else {
+			int v = cCodec->bitstreamVersion();
+			if (v == g.iCodecAlpha)
+				umtType = MessageHandler::UDPVoiceCELTAlpha;
+			else if (v == g.iCodecBeta)
+				umtType = MessageHandler::UDPVoiceCELTBeta;
+			else {
+				qWarning() << "Couldn't find message type for codec version" << v;
+			}
+		}
+	}
+
+	if (umtType != previousType) {
+		iBufferedFrames = 0;
+		qlFrames.clear();
+		opusBuffer.clear();
+	}
+
+	return true;
+}
+
+int AudioInput::encodeOpusFrame(short *source, int size, unsigned char *buffer) {
+	int len = 0;
+#ifdef USE_OPUS
+	if (!bPreviousVoice)
+		opus_encoder_ctl(opusState, OPUS_RESET_STATE, NULL);
+
+	opus_encoder_ctl(opusState, OPUS_SET_BITRATE(iAudioQuality));
+
+	len = opus_encode(opusState, source, size, buffer, 512);
+	const int tenMsFrameCount = (size / iFrameSize);
+	iBitrate = (len * 100 * 8) / tenMsFrameCount;
+#endif
+	return len;
+}
+
+int AudioInput::encodeCELTFrame(short *psSource, unsigned char *buffer) {
+	int len = 0;
+	if (!cCodec)
+		return len;
+
+	if (!bPreviousVoice)
+		cCodec->celt_encoder_ctl(ceEncoder, CELT_RESET_STATE);
+
+	cCodec->celt_encoder_ctl(ceEncoder, CELT_SET_PREDICTION(0));
+
+	cCodec->celt_encoder_ctl(ceEncoder, CELT_SET_VBR_RATE(iAudioQuality));
+	len = cCodec->encode(ceEncoder, psSource, buffer, qMin(iAudioQuality / (8 * 100), 127));
+	iBitrate = len * 100 * 8;
+
+	return len;
 }
- */
 
 void AudioInput::encodeAudioFrame() {
 	int iArg;
@@ -628,47 +738,7 @@ void AudioInput::encodeAudioFrame() {
 	}
 
 	QMutexLocker l(&qmSpeex);
-
-	if (bResetProcessor) {
-		if (sppPreprocess)
-			speex_preprocess_state_destroy(sppPreprocess);
-		if (sesEcho)
-			speex_echo_state_destroy(sesEcho);
-
-		sppPreprocess = speex_preprocess_state_init(iFrameSize, iSampleRate);
-
-		iArg = 1;
-		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_VAD, &iArg);
-		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_AGC, &iArg);
-		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_DENOISE, &iArg);
-		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_DEREVERB, &iArg);
-
-		iArg = 30000;
-		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_AGC_TARGET, &iArg);
-
-		float v = 30000.0f / static_cast<float>(g.s.iMinLoudness);
-		iArg = iroundf(floorf(20.0f * log10f(v)));
-		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_AGC_MAX_GAIN, &iArg);
-
-		iArg = -60;
-		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_AGC_DECREMENT, &iArg);
-
-		iArg = g.s.iNoiseSuppress;
-		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_NOISE_SUPPRESS, &iArg);
-
-		if (iEchoChannels > 0) {
-			sesEcho = speex_echo_state_init_mc(iFrameSize, iFrameSize*10, 1, bEchoMulti ? iEchoChannels : 1);
-			iArg = iSampleRate;
-			speex_echo_ctl(sesEcho, SPEEX_ECHO_SET_SAMPLING_RATE, &iArg);
-			speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_ECHO_STATE, sesEcho);
-
-			qWarning("AudioInput: ECHO CANCELLER ACTIVE");
-		} else {
-			sesEcho = NULL;
-		}
-
-		bResetProcessor = false;
-	}
+	resetAudioProcessor();
 
 	speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_GET_AGC_GAIN, &iArg);
 	float gainValue = static_cast<float>(iArg);
@@ -741,19 +811,28 @@ void AudioInput::encodeAudioFrame() {
 			p->setTalking(Settings::Shouting);
 	}
 
-	if (g.s.bPushClick && (g.s.atTransmit == Settings::PushToTalk)) {
+	if (g.s.bTxAudioCue && g.uiSession != 0) {
 		AudioOutputPtr ao = g.ao;
 		if (bIsSpeech && ! bPreviousVoice && ao)
-			ao->playSample(g.s.qsPushClickOn);
-		else if (ao && !bIsSpeech && bPreviousVoice && ao)
-			ao->playSample(g.s.qsPushClickOff);
+			ao->playSample(g.s.qsTxAudioCueOn);
+		else if (ao && !bIsSpeech && bPreviousVoice)
+			ao->playSample(g.s.qsTxAudioCueOff);
 	}
+
 	if (! bIsSpeech && ! bPreviousVoice) {
 		iBitrate = 0;
-		if (g.s.iIdleTime && ! g.s.bDeaf && ((tIdle.elapsed() / 1000000ULL) > g.s.iIdleTime)) {
-			emit doDeaf();
-			tIdle.restart();
+
+		if (g.s.iaeIdleAction != Settings::Nothing && ((tIdle.elapsed() / 1000000ULL) > g.s.iIdleTime)) {
+
+			if (g.s.iaeIdleAction == Settings::Deafen && !g.s.bDeaf) {
+				tIdle.restart();
+				emit doDeaf();
+			} else if (g.s.iaeIdleAction == Settings::Mute && !g.s.bMute) {
+				tIdle.restart();
+				emit doMute();
+			}
 		}
+
 		spx_int32_t increment = 0;
 		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_AGC_INCREMENT, &increment);
 		return;
@@ -763,87 +842,45 @@ void AudioInput::encodeAudioFrame() {
 	}
 
 	tIdle.restart();
-	/*
-		int r = celt_encoder_ctl(ceEncoder, CELT_SET_POST_MDCT_CALLBACK(celtBack, NULL));
-		qWarning() << "Set Callback" << r;
-	*/
 
 	unsigned char buffer[512];
 	int len;
 
-	if (umtType != MessageHandler::UDPVoiceSpeex) {
-		CELTCodec *switchto = NULL;
+	bool encoded = true;
+	if (!selectCodec())
+		return;
 
-		if ((! g.uiSession || (g.s.lmLoopMode == Settings::Local)) && (! g.qmCodecs.isEmpty())) {
-			// Use latest for local loopback
-			QMap<int, CELTCodec *>::const_iterator i = g.qmCodecs.constEnd();
-			--i;
-			switchto = i.value();
-		} else {
-			if (cCodec && bPreviousVoice) {
-				// Currently talking, don't switch unless you must.
-				int v = cCodec->bitstreamVersion();
-				if ((v == g.iCodecAlpha) || (v == g.iCodecBeta))
-					switchto = cCodec;
-			}
-		}
-		if (! switchto) {
-			switchto = g.qmCodecs.value(g.bPreferAlpha ? g.iCodecAlpha : g.iCodecBeta);
-			if (! switchto) {
-				switchto = g.qmCodecs.value(g.bPreferAlpha ? g.iCodecBeta : g.iCodecAlpha);
-			}
-		}
-		if (switchto != cCodec) {
-			if (cCodec && ceEncoder) {
-				cCodec->celt_encoder_destroy(ceEncoder);
-				ceEncoder = NULL;
-			}
-			cCodec = switchto;
-			if (cCodec) {
-				ceEncoder = cCodec->encoderCreate();
-			}
-		} else if (cCodec && ! bPreviousVoice) {
-			cCodec->celt_encoder_ctl(ceEncoder, CELT_RESET_STATE);
-		}
-
-		if (! cCodec)
+	if (umtType == MessageHandler::UDPVoiceCELTAlpha || umtType == MessageHandler::UDPVoiceCELTBeta) {
+		len = encodeCELTFrame(psSource, buffer);
+		if (len == 0)
 			return;
+		++iBufferedFrames;
+	} else if (umtType == MessageHandler::UDPVoiceOpus) {
+		encoded = false;
+		opusBuffer.insert(opusBuffer.end(), psSource, psSource + iFrameSize);
+		++iBufferedFrames;
 
-		if (g.uiSession && g.s.lmLoopMode != Settings::Local) {
-			int v = cCodec->bitstreamVersion();
-			if (v == g.iCodecAlpha)
-				umtType = MessageHandler::UDPVoiceCELTAlpha;
-			else if (v == g.iCodecBeta)
-				umtType = MessageHandler::UDPVoiceCELTBeta;
-			else {
-				qWarning() << "Couldn't find message type for codec version" << v;
+		if (!bIsSpeech || iBufferedFrames >= iAudioFrames) {
+			if (iBufferedFrames < iAudioFrames) {
+				// Stuff frame to framesize if speech ends and we don't have enough audio
+				const size_t missingFrames = iAudioFrames - iBufferedFrames;
+				opusBuffer.insert(opusBuffer.end(), iFrameSize * missingFrames, 0);
+				iBufferedFrames += missingFrames;
+			}
+
+			len = encodeOpusFrame(&opusBuffer[0], iBufferedFrames * iFrameSize, buffer);
+			opusBuffer.clear();
+			if (len <= 0) {
+				iBitrate = 0;
+				qWarning() << "encodeOpusFrame failed" << iBufferedFrames << iFrameSize << len;
 				return;
 			}
+			encoded = true;
 		}
-
-		cCodec->celt_encoder_ctl(ceEncoder, CELT_SET_PREDICTION(0));
-
-		cCodec->celt_encoder_ctl(ceEncoder,CELT_SET_VBR_RATE(iAudioQuality));
-		len = cCodec->encode(ceEncoder, psSource, buffer, qMin(iAudioQuality / 800, 127));
-		iBitrate = len * 100 * 8;
-	} else {
-		int vbr = 0;
-		speex_encoder_ctl(esSpeex, SPEEX_GET_VBR_MAX_BITRATE, &vbr);
-		if (vbr != iAudioQuality) {
-			vbr = iAudioQuality;
-			speex_encoder_ctl(esSpeex, SPEEX_SET_VBR_MAX_BITRATE, &vbr);
-		}
-
-		if (! bPreviousVoice)
-			speex_encoder_ctl(esSpeex, SPEEX_RESET_STATE, NULL);
-
-		speex_encode_int(esSpeex, psSource, &sbBits);
-		len = speex_bits_write(&sbBits, reinterpret_cast<char *>(buffer), 127);
-		iBitrate = len * 50 * 8;
-		speex_bits_reset(&sbBits);
 	}
 
-	flushCheck(QByteArray(reinterpret_cast<const char *>(buffer), len), ! bIsSpeech);
+	if (encoded)
+		flushCheck(QByteArray(reinterpret_cast<const char *>(buffer), len), ! bIsSpeech);
 
 	if (! bIsSpeech)
 		iBitrate = 0;
@@ -851,9 +888,24 @@ void AudioInput::encodeAudioFrame() {
 	bPreviousVoice = bIsSpeech;
 }
 
+static void sendAudioFrame(const char *data, PacketDataStream &pds) {
+	ServerHandlerPtr sh = g.sh;
+	if (sh) {
+		VoiceRecorderPtr recorder(sh->recorder);
+		if (recorder)
+			recorder->getRecordUser().addFrame(QByteArray(data, pds.size() + 1));
+	}
+
+	if (g.s.lmLoopMode == Settings::Local)
+		LoopUser::lpLoopy.addFrame(QByteArray(data, pds.size() + 1));
+	else if (sh)
+		sh->sendMessage(data, pds.size() + 1);
+}
+
 void AudioInput::flushCheck(const QByteArray &frame, bool terminator) {
 	qlFrames << frame;
-	if (! terminator && qlFrames.count() < iAudioFrames)
+
+	if (! terminator && iBufferedFrames < iAudioFrames)
 		return;
 
 	int flags = g.iTarget;
@@ -868,19 +920,34 @@ void AudioInput::flushCheck(const QByteArray &frame, bool terminator) {
 	char data[1024];
 	data[0] = static_cast<unsigned char>(flags);
 
+	int frames = iBufferedFrames;
+	iBufferedFrames = 0;
+
 	PacketDataStream pds(data + 1, 1023);
-	pds << iFrameCounter - qlFrames.count();
+	// Sequence number
+	pds << iFrameCounter - frames;
 
-	if (terminator)
-		qlFrames << QByteArray();
-
-	for (int i=0;i<qlFrames.count(); ++i) {
-		const QByteArray &qba = qlFrames.at(i);
-		unsigned char head = static_cast<unsigned char>(qba.size());
-		if (i < qlFrames.count() - 1)
-			head |= 0x80;
-		pds.append(head);
+	if (umtType == MessageHandler::UDPVoiceOpus) {
+		const QByteArray &qba = qlFrames.takeFirst();
+		int size = qba.size();
+		if (terminator)
+			size |= 1 << 13;
+		pds << size;
 		pds.append(qba.constData(), qba.size());
+	} else {
+		if (terminator) {
+			qlFrames << QByteArray();
+			++frames;
+		}
+
+		for (int i = 0; i < frames; ++i) {
+			const QByteArray &qba = qlFrames.takeFirst();
+			unsigned char head = static_cast<unsigned char>(qba.size());
+			if (i < frames - 1)
+				head |= 0x80;
+			pds.append(head);
+			pds.append(qba.constData(), qba.size());
+		}
 	}
 
 	if (g.s.bTransmitPosition && g.p && ! g.bCenterPosition && g.p->fetch()) {
@@ -889,19 +956,9 @@ void AudioInput::flushCheck(const QByteArray &frame, bool terminator) {
 		pds << g.p->fPosition[2];
 	}
 
-	ServerHandlerPtr sh = g.sh;
-	if (sh) {
-		VoiceRecorderPtr recorder(sh->recorder);
-		if (recorder)
-			recorder->getRecordUser().addFrame(QByteArray(data, pds.size() + 1));
-	}
+	sendAudioFrame(data, pds);
 
-	if (g.s.lmLoopMode == Settings::Local)
-		LoopUser::lpLoopy.addFrame(QByteArray(data, pds.size() + 1));
-	else if (sh)
-		sh->sendMessage(data, pds.size() + 1);
-
-	qlFrames.clear();
+	Q_ASSERT(qlFrames.isEmpty());
 }
 
 bool AudioInput::isAlive() const {
