@@ -1,5 +1,5 @@
-/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
-   Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
+   Copyright (C) 2009-2011, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
 
@@ -29,13 +29,16 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifndef _GLOBAL_H
-#define _GLOBAL_H
+#ifndef GLOBAL_H_
+#define GLOBAL_H_
 
+#include <boost/shared_ptr.hpp>
+#include <QtCore/QDir>
+
+#include "ACL.h"
 #include "Settings.h"
 #include "Timer.h"
 #include "Version.h"
-#include "ACL.h"
 
 // Global helper class to spread variables around across threads.
 
@@ -50,8 +53,11 @@ class QSettings;
 class Overlay;
 class LCD;
 class BonjourClient;
+class OverlayClient;
 class CELTCodec;
 
+class QNetworkAccessManager;
+
 struct Global {
 private:
 	Q_DISABLE_COPY(Global)
@@ -59,7 +65,7 @@ public:
 	static Global *g_global_struct;
 	MainWindow *mw;
 	Settings s;
-	ServerHandler *sh;
+	boost::shared_ptr<ServerHandler> sh;
 	boost::shared_ptr<AudioInput> ai;
 	boost::shared_ptr<AudioOutput> ao;
 	Database *db;
@@ -78,20 +84,28 @@ public:
 	bool bPushToMute;
 	bool bCenterPosition;
 	bool bPosTest;
+	bool bInAudioWizard;
+	OverlayClient *ocIntercept;
 	int iAudioPathTime;
 	unsigned int uiSession;
 	ChanACL::Permissions pPermissions;
 	int iMaxBandwidth;
 	int iAudioBandwidth;
 	QDir qdBasePath;
-	QString qsRegionalHost;
 	QMap<int, CELTCodec *> qmCodecs;
 	int iCodecAlpha, iCodecBeta;
 	bool bPreferAlpha;
+	bool bOpus;
 	bool bAttenuateOthers;
 	bool bAllowHTML;
 	unsigned int uiMessageLength;
 	unsigned int uiImageLength;
+	bool bQuit;
+	QString qsCurrentStyle;
+
+	bool bHappyEaster;
+	static const char ccHappyEaster[];
+
 	Global();
 	~Global();
 };
