@@ -1,5 +1,5 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
-   Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
+   Copyright (C) 2009-2011, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
 
@@ -29,9 +29,12 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#include "mumble_pch.hpp"
+
+#include "LookConfig.h"
+
 #include "AudioInput.h"
 #include "AudioOutput.h"
-#include "LookConfig.h"
 #include "Global.h"
 #include "MainWindow.h"
 
@@ -44,6 +47,11 @@ static ConfigRegistrar registrar(1100, LookConfigNew);
 LookConfig::LookConfig(Settings &st) : ConfigWidget(st) {
 	setupUi(this);
 
+#ifndef Q_OS_MAC
+	if (! QSystemTrayIcon::isSystemTrayAvailable())
+#endif
+		qgbTray->hide();
+
 	qcbLanguage->addItem(tr("System default"));
 	QDir d(QLatin1String(":"),QLatin1String("mumble_*.qm"),QDir::Name,QDir::Files);
 	QStringList langs;
@@ -118,12 +126,14 @@ void LookConfig::load(const Settings &r) {
 	qleCSS->setText(r.qsSkin);
 	loadComboBox(qcbExpand, r.ceExpand);
 	loadComboBox(qcbChannelDrag, r.ceChannelDrag);
-	loadCheckBox(qcbShowChatbar, r.bShowChatbar);
 	loadCheckBox(qcbUsersTop, r.bUserTop);
 	loadCheckBox(qcbAskOnQuit, r.bAskOnQuit);
-	loadCheckBox(qcbHideTray, r.bHideTray);
+	loadCheckBox(qcbHideTray, r.bHideInTray);
 	loadCheckBox(qcbStateInTray, r.bStateInTray);
 	loadCheckBox(qcbShowUserCount, r.bShowUserCount);
+	loadCheckBox(qcbShowContextMenuInMenuBar, r.bShowContextMenuInMenuBar);
+	loadCheckBox(qcbHighContrast, r.bHighContrast);
+	loadCheckBox(qcbChatBarUseSelection, r.bChatBarUseSelection);
 }
 
 void LookConfig::save() const {
@@ -155,22 +165,27 @@ void LookConfig::save() const {
 
 	s.ceExpand=static_cast<Settings::ChannelExpand>(qcbExpand->currentIndex());
 	s.ceChannelDrag=static_cast<Settings::ChannelDrag>(qcbChannelDrag->currentIndex());
-	s.bShowChatbar = qcbShowChatbar->isChecked();
 	s.bUserTop=qcbUsersTop->isChecked();
 	s.aotbAlwaysOnTop = static_cast<Settings::AlwaysOnTopBehaviour>(qcbAlwaysOnTop->currentIndex());
 	s.bAskOnQuit = qcbAskOnQuit->isChecked();
-	s.bHideTray = qcbHideTray->isChecked();
+	s.bHideInTray = qcbHideTray->isChecked();
 	s.bStateInTray = qcbStateInTray->isChecked();
 	s.bShowUserCount = qcbShowUserCount->isChecked();
+	s.bShowContextMenuInMenuBar = qcbShowContextMenuInMenuBar->isChecked();
+	s.bHighContrast = qcbHighContrast->isChecked();
+	s.bChatBarUseSelection = qcbChatBarUseSelection->isChecked();
 }
 
 void LookConfig::accept() const {
-	if (! s.qsStyle.isEmpty()) {
+	if (! s.qsStyle.isEmpty() && g.qsCurrentStyle != s.qsStyle) {
 		qApp->setStyle(s.qsStyle);
+		g.qsCurrentStyle = s.qsStyle;
 	}
 	if (s.qsSkin.isEmpty()) {
-		qApp->setStyleSheet(MainWindow::defaultStyleSheet);
-		g.mw->qteLog->document()->setDefaultStyleSheet(qApp->styleSheet());
+		if (qApp->styleSheet() != MainWindow::defaultStyleSheet) {
+			qApp->setStyleSheet(MainWindow::defaultStyleSheet);
+			g.mw->qteLog->document()->setDefaultStyleSheet(qApp->styleSheet());
+		}
 	} else {
 		QFile file(s.qsSkin);
 		file.open(QFile::ReadOnly);
@@ -180,6 +195,7 @@ void LookConfig::accept() const {
 			g.mw->qteLog->document()->setDefaultStyleSheet(sheet);
 		}
 	}
+	g.mw->setShowDockTitleBars(g.s.wlWindowLayout == Settings::LayoutCustom);
 }
 
 bool LookConfig::expert(bool b) {
@@ -189,6 +205,7 @@ bool LookConfig::expert(bool b) {
 	qcbStyle->setVisible(b);
 	qliStyle->setVisible(b);
 	qcbStateInTray->setVisible(b);
+	qcbShowContextMenuInMenuBar->setVisible(b);
 	return true;
 }
 
