@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -28,17 +28,22 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#include "mumble_pch.hpp"
+
 #include "Overlay.h"
-#include "OverlayText.h"
-#include "User.h"
+
 #include "Channel.h"
-#include "Global.h"
-#include "Message.h"
+#include "ClientUser.h"
 #include "Database.h"
-#include "NetworkConfig.h"
-#include "ServerHandler.h"
-#include "MainWindow.h"
+#include "Global.h"
 #include "GlobalShortcut.h"
+#include "MainWindow.h"
+#include "Message.h"
+#include "OverlayText.h"
+#include "RichTextEditor.h"
+#include "ServerHandler.h"
+#include "User.h"
+#include "WebFetch.h"
 
 OverlayAppInfo::OverlayAppInfo(QString name, QIcon icon) {
 	qsDisplayName = name;
@@ -97,7 +102,7 @@ Overlay::Overlay() : QObject() {
 #endif
 
 	if (! qlsServer->listen(pipepath)) {
-		QMessageBox::warning(NULL, QLatin1String("Mumble"), tr("Failed to create communication with overlay at %2: %1. No overlay will be available.").arg(qlsServer->errorString(),pipepath), QMessageBox::Ok, QMessageBox::NoButton);
+		QMessageBox::warning(NULL, QLatin1String("Mumble"), tr("Failed to create communication with overlay at %2: %1. No overlay will be available.").arg(Qt::escape(qlsServer->errorString()), Qt::escape(pipepath)), QMessageBox::Ok, QMessageBox::NoButton);
 	} else {
 		qWarning() << "Overlay: Listening on" << qlsServer->fullServerName();
 		connect(qlsServer, SIGNAL(newConnection()), this, SLOT(newConnection()));
@@ -199,7 +204,7 @@ void Overlay::verifyTexture(ClientUser *cp, bool allowupdate) {
 	if (! cp->qbaTexture.isEmpty()) {
 		bool valid = true;
 
-		if (cp->qbaTexture.length() < sizeof(unsigned int)) {
+		if (cp->qbaTexture.length() < static_cast<int>(sizeof(unsigned int))) {
 			valid = false;
 		} else if (qFromBigEndian<unsigned int>(reinterpret_cast<const unsigned char *>(cp->qbaTexture.constData())) == 600 * 60 * 4) {
 			QByteArray qba = qUncompress(cp->qbaTexture);
@@ -268,15 +273,21 @@ void Overlay::verifyTexture(ClientUser *cp, bool allowupdate) {
 			qb.open(QIODevice::ReadOnly);
 
 			QImageReader qir;
-			if (cp->qbaTexture.startsWith("<?xml"))
-				qir.setFormat("svg");
-			qir.setDevice(&qb);
-			if (! qir.canRead() || (qir.size().width() > 1024) || (qir.size().height() > 1024)) {
-				valid = false;
+			qir.setAutoDetectImageFormat(false);
+
+			QByteArray fmt;
+			if (RichTextImage::isValidImage(cp->qbaTexture, fmt)) {
+				qir.setFormat(fmt);
+				qir.setDevice(&qb);
+				if (! qir.canRead() || (qir.size().width() > 1024) || (qir.size().height() > 1024)) {
+					valid = false;
+				} else {
+					cp->qbaTextureFormat = qir.format();
+					QImage qi = qir.read();
+					valid = ! qi.isNull();
+				}
 			} else {
-				cp->qbaTextureFormat = qir.format();
-				QImage qi = qir.read();
-				valid = ! qi.isNull();
+				valid = false;
 			}
 		}
 		if (! valid) {
@@ -328,110 +339,3 @@ void Overlay::requestTexture(ClientUser *cu) {
 			verifyTexture(cu, false);
 	}
 }
-
-void Overlay::checkUpdates() {
-#ifndef Q_OS_MAC
-	return;
-#endif
-
-	QUrl url;
-	url.setScheme(QLatin1String("http"));
-	url.setHost(g.qsRegionalHost);
-	url.setPath(QLatin1String("/overlay.php"));
-
-	url.addQueryItem(QLatin1String("ver"), QLatin1String(QUrl::toPercentEncoding(QLatin1String(MUMBLE_RELEASE))));
-#if defined(Q_OS_WIN)
-	url.addQueryItem(QLatin1String("os"), QLatin1String("Win32"));
-#elif defined(Q_OS_MAC)
-	url.addQueryItem(QLatin1String("os"), QLatin1String("MacOSX"));
-#else
-	url.addQueryItem(QLatin1String("os"), QLatin1String("Unix"));
-#endif
-
-#ifdef QT_NO_DEBUG
-	QNetworkReply *rep = Network::get(url);
-	connect(rep, SIGNAL(finished()), this, SLOT(finished()));
-#else
-	g.mw->msgBox(tr("Skipping overlay update in debug mode."));
-#endif
-}
-
-void Overlay::finished() {
-	QNetworkReply *rep = qobject_cast<QNetworkReply *>(sender());
-	QUrl url = rep->request().url();
-
-	if (rep->error() == QNetworkReply::NoError) {
-		const QString &path = url.path();
-		if (path == QLatin1String("/overlay.php")) {
-			qmOverlayHash.clear();
-			QDomDocument doc;
-			doc.setContent(rep->readAll());
-
-			QDomElement root=doc.documentElement();
-			QDomNode n = root.firstChild();
-			while (!n.isNull()) {
-				QDomElement e = n.toElement();
-				if (!e.isNull()) {
-					if (e.tagName() == QLatin1String("file")) {
-						QString name = QFileInfo(e.attribute(QLatin1String("name"))).fileName();
-						QString hash = e.attribute(QLatin1String("hash"));
-						qmOverlayHash.insert(name, hash);
-					}
-				}
-				n = n.nextSibling();
-			}
-
-#ifdef Q_OS_MAC
-			QMap<QString, QString>::const_iterator i;
-			for (i = qmOverlayHash.constBegin(); i != qmOverlayHash.constEnd(); ++i) {
-				QString val = i.value();
-				QString key = i.key();
-				if (! val.isEmpty() && key.startsWith(QLatin1String("MumbleOverlay")) && key.endsWith(QLatin1String(".pkg"))) {
-					bool update = true;
-					QFile f(g.qdBasePath.absolutePath() + QLatin1String("/Overlay/MumbleOverlay.pkg"));
-					if (f.exists() && f.open(QIODevice::ReadOnly)) {
-						QString h = QLatin1String(sha1(f.readAll()).toHex());
-						if (val == qmOverlayHash.value(QLatin1String("MumbleOverlay.pkg")))
-							update = false;
-					}
-					if (update) {
-						QUrl url;
-						url.setScheme(QLatin1String("http"));
-						url.setHost(rep->url().host());
-						url.setPath(QString::fromLatin1("overlay/%1").arg(key));
-
-						QNetworkReply *r = Network::get(url);
-						connect(r, SIGNAL(finished()), this, SLOT(finished()));
-					}
-				}
-			}
-#endif
-		} else {
-			QString fname = QFileInfo(path).fileName();
-			if (qmOverlayHash.contains(fname)) {
-				QByteArray qba = rep->readAll();
-				if (qmOverlayHash.value(fname) == QLatin1String(sha1(qba).toHex())) {
-#ifdef Q_OS_MAC
-					if (fname.startsWith(QLatin1String("MumbleOverlay")) && fname.endsWith(QLatin1String(".pkg"))) {
-						QFile f;
-						f.setFileName(g.qdBasePath.absolutePath() + QLatin1String("/Overlay/MumbleOverlay.pkg"));
-						if (f.open(QIODevice::WriteOnly)) {
-							f.write(qba);
-							f.close();
-							g.mw->msgBox(tr("Downloaded new or updated overlay support file to %1.").arg(f.fileName()));
-						}
-					}
-#endif
-				}
-			}
-		}
-	} else {
-		if (url.host() == g.qsRegionalHost) {
-			url.setHost(QLatin1String("mumble.info"));
-			QNetworkReply *nrep = Network::get(url);
-			connect(nrep, SIGNAL(finished()), this, SLOT(finished()));
-		}
-	}
-
-	rep->deleteLater();
-}
