@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -28,10 +28,11 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifndef _USERVIEW_H
-#define _USERVIEW_H
+#ifndef USERVIEW_H_
+#define USERVIEW_H_
 
-#include "mumble_pch.hpp"
+#include <QtGui/QStyledItemDelegate>
+#include <QtGui/QTreeView>
 
 #include "Timer.h"
 
@@ -49,9 +50,10 @@ class UserDelegate : public QStyledItemDelegate {
 class UserView : public QTreeView {
 	private:
 		Q_OBJECT
-		Q_DISABLE_COPY(UserView);
+		Q_DISABLE_COPY(UserView)
 	protected:
 		void mouseReleaseEvent(QMouseEvent *);
+		void keyPressEvent(QKeyEvent *);
 		bool event(QEvent *);
 
 		QTimer *qtSearch;
@@ -62,11 +64,9 @@ class UserView : public QTreeView {
 		UserView(QWidget *);
 		void keyboardSearch(const QString &search);
 	public slots:
-		void contextMenu(const QPoint &pos);
-		void doubleClick(const QModelIndex &idx);
+		void nodeActivated(const QModelIndex &idx);
 		void selectSearchResult();
+
 };
 
-#else
-class UserView;
 #endif
