@@ -1,5 +1,5 @@
-/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
-   Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
+   Copyright (C) 2009-2011, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
 
@@ -29,6 +29,8 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#include "murmur_pch.h"
+
 #include "User.h"
 #include "Channel.h"
 #include "ACL.h"
@@ -38,7 +40,7 @@
 #include "Connection.h"
 #include "Server.h"
 #include "ServerUser.h"
-#include "DBus.h"
+#include "Version.h"
 
 #define MSG_SETUP(st) \
 	if (uSource->sState != st) { \
@@ -119,18 +121,18 @@ void Server::msgAuthenticate(ServerUser *uSource, MumbleProto::Authenticate &msg
 	// Fetch ID and stored username.
 	// Since this may call DBus, which may recall our dbus messages, this function needs
 	// to support re-entrancy, and also to support the fact that sessions may go away.
-	int id = authenticate(uSource->qsName, pw, uSource->qslEmail, uSource->qsHash, uSource->bVerified, uSource->peerCertificateChain());
+	int id = authenticate(uSource->qsName, pw, uSource->uiSession, uSource->qslEmail, uSource->qsHash, uSource->bVerified, uSource->peerCertificateChain());
 
 	uSource->iId = id >= 0 ? id : -1;
 
 	QString reason;
 	MumbleProto::Reject_RejectType rtType = MumbleProto::Reject_RejectType_None;
 
-	if (! nameok && (uSource->iId == -1)) {
+	if (id==-2 && ! nameok) {
 		reason = "Invalid username";
 		rtType = MumbleProto::Reject_RejectType_InvalidUsername;
 	} else if (id==-1) {
-		reason = "Wrong password for user";
+		reason = "Wrong certificate or password for existing user";
 		rtType = MumbleProto::Reject_RejectType_WrongUserPW;
 	} else if (id==-2 && ! qsPassword.isEmpty() && qsPassword != pw) {
 		reason = "Invalid server password";
@@ -198,20 +200,28 @@ void Server::msgAuthenticate(ServerUser *uSource, MumbleProto::Authenticate &msg
 	mpcrypt.set_client_nonce(std::string(reinterpret_cast<const char *>(uSource->csCrypt.decrypt_iv), AES_BLOCK_SIZE));
 	sendMessage(uSource, mpcrypt);
 
+	bool fake_celt_support = false;
 	if (msg.celt_versions_size() > 0) {
 		for (int i=0;i < msg.celt_versions_size(); ++i)
 			uSource->qlCodecs.append(msg.celt_versions(i));
 	} else {
-		uSource->qlCodecs.append(static_cast<qint32>(0x8000000a));
+		uSource->qlCodecs.append(static_cast<qint32>(0x8000000b));
+		fake_celt_support = true;
 	}
-	recheckCodecVersions();
+	uSource->bOpus = msg.opus();
+	recheckCodecVersions(uSource);
 
 	MumbleProto::CodecVersion mpcv;
 	mpcv.set_alpha(iCodecAlpha);
 	mpcv.set_beta(iCodecBeta);
 	mpcv.set_prefer_alpha(bPreferAlpha);
+	mpcv.set_opus(bOpus);
 	sendMessage(uSource, mpcv);
 
+	if (!bOpus && uSource->bOpus && fake_celt_support) {
+		sendTextMessage(NULL, uSource, false, QLatin1String("<strong>WARNING:</strong> Your client doesn't support the CELT codec, you won't be able to talk to or hear most clients. Please make sure your client was built with CELT support."));
+	}
+
 	// Transmit channel tree
 	QQueue<Channel *> q;
 	QSet<Channel *> chans;
@@ -259,7 +269,12 @@ void Server::msgAuthenticate(ServerUser *uSource, MumbleProto::Authenticate &msg
 	// Transmit user profile
 	MumbleProto::UserState mpus;
 
-	Channel *lc = qhChannels.value(readLastChannel(uSource->iId));
+	Channel *lc;
+	if (bRememberChan)
+		lc = qhChannels.value(readLastChannel(uSource->iId));
+	else
+		lc = qhChannels.value(iDefaultChan);
+
 	if (! lc || ! hasPermission(uSource, lc, ChanACL::Enter)) {
 		lc = qhChannels.value(iDefaultChan);
 		if (! lc || ! hasPermission(uSource, lc, ChanACL::Enter)) {
@@ -333,6 +348,10 @@ void Server::msgAuthenticate(ServerUser *uSource, MumbleProto::Authenticate &msg
 			mpus.set_mute(true);
 		if (u->bSuppress)
 			mpus.set_suppress(true);
+		if (u->bPrioritySpeaker)
+			mpus.set_priority_speaker(true);
+		if (u->bRecording)
+			mpus.set_recording(true);
 		if (u->bSelfDeaf)
 			mpus.set_self_deaf(true);
 		else if (u->bSelfMute)
@@ -358,7 +377,7 @@ void Server::msgAuthenticate(ServerUser *uSource, MumbleProto::Authenticate &msg
 		mpss.set_permissions(ChanACL::All);
 	} else {
 		QMutexLocker qml(&qmCache);
-		ChanACL::hasPermission(uSource, root, ChanACL::Enter, acCache);
+		ChanACL::hasPermission(uSource, root, ChanACL::Enter, &acCache);
 		mpss.set_permissions(acCache.value(uSource)->value(root));
 	}
 
@@ -370,6 +389,17 @@ void Server::msgAuthenticate(ServerUser *uSource, MumbleProto::Authenticate &msg
 	mpsc.set_image_message_length(iMaxImageMessageLength);
 	sendMessage(uSource, mpsc);
 
+	MumbleProto::SuggestConfig mpsug;
+	if (! qvSuggestVersion.isNull())
+		mpsug.set_version(qvSuggestVersion.toUInt());
+	if (! qvSuggestPositional.isNull())
+		mpsug.set_positional(qvSuggestPositional.toBool());
+	if (! qvSuggestPushToTalk.isNull())
+		mpsug.set_push_to_talk(qvSuggestPushToTalk.toBool());
+	if (mpsug.ByteSize() > 0) {
+		sendMessage(uSource, mpsug);
+	}
+
 	log(uSource, "Authenticated");
 
 	emit userConnected(uSource);
@@ -407,10 +437,12 @@ void Server::msgBanList(ServerUser *uSource, MumbleProto::BanList &msg) {
 			b.qsUsername = u8(be.name());
 			b.qsHash = u8(be.hash());
 			b.qsReason = u8(be.reason());
-			if (be.has_start())
+			if (be.has_start()) {
 				b.qdtStart = QDateTime::fromString(u8(be.start()), Qt::ISODate);
-			else
+				b.qdtStart.setTimeSpec(Qt::UTC);
+			} else {
 				b.qdtStart = QDateTime::currentDateTime().toUTC();
+			}
 			b.iDuration = be.duration();
 			if (b.isValid())
 				qlBans << b;
@@ -444,9 +476,11 @@ void Server::msgUserState(ServerUser *uSource, MumbleProto::UserState &msg) {
 	MSG_SETUP(ServerUser::Authenticated);
 	VICTIM_SETUP;
 
-	bool bNoBroadcast = false;
 	Channel *root = qhChannels.value(0);
 
+	/*
+		First check all permissions involved
+	*/
 	if ((pDstServerUser->iId == 0) && (uSource->iId != 0)) {
 		PERM_DENIED_TYPE(SuperUser);
 		return;
@@ -470,16 +504,20 @@ void Server::msgUserState(ServerUser *uSource, MumbleProto::UserState &msg) {
 			return;
 		}
 		if (iMaxUsersPerChannel && (c->qlUsers.count() >= iMaxUsersPerChannel)) {
-			PERM_DENIED_FALLBACK(ChannelFull, 0x010201, QLatin1String("Channel is full."));
+			PERM_DENIED_FALLBACK(ChannelFull, 0x010201, QLatin1String("Channel is full"));
 			return;
 		}
 	}
 
-	if (msg.has_mute() || msg.has_deaf() || msg.has_suppress()) {
+	if (msg.has_mute() || msg.has_deaf() || msg.has_suppress() || msg.has_priority_speaker()) {
 		if (pDstServerUser->iId == 0) {
 			PERM_DENIED_TYPE(SuperUser);
 			return;
 		}
+		if (uSource->cChannel->bTemporary) {
+			PERM_DENIED_TYPE(TemporaryChannel);
+			return;
+		}
 		if (! hasPermission(uSource, pDstServerUser->cChannel, ChanACL::MuteDeafen) || msg.suppress()) {
 			PERM_DENIED(uSource, pDstServerUser->cChannel, ChanACL::MuteDeafen);
 			return;
@@ -512,7 +550,7 @@ void Server::msgUserState(ServerUser *uSource, MumbleProto::UserState &msg) {
 	}
 
 	if (msg.has_texture()) {
-		if (iMaxImageMessageLength && (msg.texture().length() > iMaxImageMessageLength)) {
+		if (iMaxImageMessageLength > 0 && (msg.texture().length() > static_cast<unsigned int>(iMaxImageMessageLength))) {
 			PERM_DENIED_TYPE(TextTooLong);
 			return;
 		}
@@ -531,25 +569,35 @@ void Server::msgUserState(ServerUser *uSource, MumbleProto::UserState &msg) {
 		}
 	}
 
-	if ((pDstServerUser != uSource) && (msg.has_self_deaf() || msg.has_self_mute() || msg.has_texture() || msg.has_plugin_context() || msg.has_plugin_identity()))
+	// Prevent self-targeting state changes from being applied to others
+	if ((pDstServerUser != uSource) && (msg.has_self_deaf() || msg.has_self_mute() || msg.has_texture() || msg.has_plugin_context() || msg.has_plugin_identity() || msg.has_recording()))
 		return;
 
-	// Permission checks done. Now enact this.
+	/*
+		-------------------- Permission checks done. Now act --------------------
+	*/
+	bool bBroadcast = false;
 
 	if (msg.has_texture()) {
 		QByteArray qba = blob(msg.texture());
 		if (uSource->iId > 0) {
+			// For registered users store the texture we just received in the database
 			if (! setTexture(uSource->iId, qba))
 				return;
 		} else {
+			// For unregistered users or SuperUser only get the hash
 			hashAssign(uSource->qbaTexture, uSource->qbaTextureHash, qba);
 		}
+
+		// The texture will be sent out later in this function
+		bBroadcast = true;
 	}
 
 	if (msg.has_self_deaf()) {
 		uSource->bSelfDeaf = msg.self_deaf();
 		if (uSource->bSelfDeaf)
 			msg.set_self_mute(true);
+		bBroadcast = true;
 	}
 
 	if (msg.has_self_mute()) {
@@ -558,16 +606,19 @@ void Server::msgUserState(ServerUser *uSource, MumbleProto::UserState &msg) {
 			msg.set_self_deaf(false);
 			uSource->bSelfDeaf = false;
 		}
+		bBroadcast = true;
 	}
 
 	if (msg.has_plugin_context()) {
-		bNoBroadcast = true;
 		uSource->ssContext = msg.plugin_context();
+		// Make sure to clear this from the packet so we don't broadcast it
+		msg.clear_plugin_context();
 	}
 
 	if (msg.has_plugin_identity()) {
-		bNoBroadcast = true;
 		uSource->qsIdentity = u8(msg.plugin_identity());
+		// Make sure to clear this from the packet so we don't broadcast it
+		msg.clear_plugin_identity();
 	}
 
 	if (! comment.isNull()) {
@@ -578,11 +629,12 @@ void Server::msgUserState(ServerUser *uSource, MumbleProto::UserState &msg) {
 			info.insert(ServerDB::User_Comment, pDstServerUser->qsComment);
 			setInfo(pDstServerUser->iId, info);
 		}
+		bBroadcast = true;
 	}
 
 
 
-	if (msg.has_mute() || msg.has_deaf() || msg.has_suppress()) {
+	if (msg.has_mute() || msg.has_deaf() || msg.has_suppress() || msg.has_priority_speaker()) {
 		if (msg.has_deaf()) {
 			pDstServerUser->bDeaf = msg.deaf();
 			if (pDstServerUser->bDeaf)
@@ -598,10 +650,31 @@ void Server::msgUserState(ServerUser *uSource, MumbleProto::UserState &msg) {
 		if (msg.has_suppress())
 			pDstServerUser->bSuppress = msg.suppress();
 
-		log(uSource, QString("Changed speak-state of %1 (%2 %3 %4)").arg(QString(*pDstServerUser),
+		if (msg.has_priority_speaker())
+			pDstServerUser->bPrioritySpeaker = msg.priority_speaker();
+
+		log(uSource, QString("Changed speak-state of %1 (%2 %3 %4 %5)").arg(QString(*pDstServerUser),
 		        QString::number(pDstServerUser->bMute),
 		        QString::number(pDstServerUser->bDeaf),
-		        QString::number(pDstServerUser->bSuppress)));
+		        QString::number(pDstServerUser->bSuppress),
+		        QString::number(pDstServerUser->bPrioritySpeaker)));
+
+		bBroadcast = true;
+	}
+
+	if (msg.has_recording() && (pDstServerUser->bRecording != msg.recording())) {
+		pDstServerUser->bRecording = msg.recording();
+
+		MumbleProto::TextMessage mptm;
+		mptm.add_tree_id(0);
+		if (pDstServerUser->bRecording)
+			mptm.set_message(u8(QString(QLatin1String("User '%1' started recording")).arg(pDstServerUser->qsName)));
+		else
+			mptm.set_message(u8(QString(QLatin1String("User '%1' stopped recording")).arg(pDstServerUser->qsName)));
+
+		sendAll(mptm, ~ 0x010203);
+
+		bBroadcast = true;
 	}
 
 	if (msg.has_channel_id()) {
@@ -609,9 +682,12 @@ void Server::msgUserState(ServerUser *uSource, MumbleProto::UserState &msg) {
 
 		userEnterChannel(pDstServerUser, c, msg);
 		log(uSource, QString("Moved %1 to %2").arg(QString(*pDstServerUser), QString(*c)));
+		bBroadcast = true;
 	}
 
+	bool bDstAclChanged = false;
 	if (msg.has_user_id()) {
+		// Handle user (Self-)Registration
 		QMap<int, QString> info;
 
 		info.insert(ServerDB::User_Name, pDstServerUser->qsName);
@@ -621,20 +697,32 @@ void Server::msgUserState(ServerUser *uSource, MumbleProto::UserState &msg) {
 		int id = registerUser(info);
 		if (id > 0) {
 			pDstServerUser->iId = id;
+			setLastChannel(pDstServerUser);
 			msg.set_user_id(id);
+			bDstAclChanged = true;
 		} else {
+			// Registration failed
 			msg.clear_user_id();
 		}
+		bBroadcast = true;
 	}
 
-	if (! bNoBroadcast) {
+	if (bBroadcast) {
+		// Texture handling for clients < 1.2.2.
+		// Send the texture data in the message.
 		if (msg.has_texture() && (pDstServerUser->qbaTexture.length() >= 4) && (qFromBigEndian<unsigned int>(reinterpret_cast<const unsigned char *>(pDstServerUser->qbaTexture.constData())) != 600 * 60 * 4)) {
+			// This is a new style texture, don't send it because the client doesn't handle it correctly / crashes.
 			msg.clear_texture();
 			sendAll(msg, ~ 0x010202);
 			msg.set_texture(blob(pDstServerUser->qbaTexture));
 		} else {
+			// This is an old style texture, empty texture or there was no texture in this packet,
+			// send the message unchanged.
 			sendAll(msg, ~ 0x010202);
 		}
+
+		// Texture / comment handling for clients >= 1.2.2.
+		// Send only a hash of the texture / comment text. The client will request the actual data if necessary.
 		if (msg.has_texture() && ! pDstServerUser->qbaTextureHash.isEmpty()) {
 			msg.clear_texture();
 			msg.set_texture_hash(blob(pDstServerUser->qbaTextureHash));
@@ -643,7 +731,11 @@ void Server::msgUserState(ServerUser *uSource, MumbleProto::UserState &msg) {
 			msg.clear_comment();
 			msg.set_comment_hash(blob(pDstServerUser->qbaCommentHash));
 		}
+
 		sendAll(msg, 0x010202);
+
+		if (bDstAclChanged)
+			clearACLCache(pDstServerUser);
 	}
 
 	emit userStateChanged(pDstServerUser);
@@ -748,6 +840,16 @@ void Server::msgChannelState(ServerUser *uSource, MumbleProto::ChannelState &msg
 		}
 	}
 
+	if(p) {
+		// Having a parent channel given means we either want to create
+		// a channel in or move a channel into this parent.
+
+		if (!canNest(p, c)) {
+			PERM_DENIED_FALLBACK(NestingLimit, 0x010204, QLatin1String("Channel nesting limit reached"));
+			return;
+		}
+	}
+
 	if (! c) {
 		// If we don't have a channel handle up to now we want to create a new channel
 		// so check if the user has enough rights and we got everything we need.
@@ -965,6 +1067,8 @@ void Server::msgTextMessage(ServerUser *uSource, MumbleProto::TextMessage &msg)
 	MSG_SETUP(ServerUser::Authenticated);
 	QMutexLocker qml(&qmCache);
 
+	TextMessage tm; // for signal userTextMessage
+
 	QSet<ServerUser *> users;
 	QQueue<Channel *> q;
 
@@ -980,9 +1084,11 @@ void Server::msgTextMessage(ServerUser *uSource, MumbleProto::TextMessage &msg)
 	if (changed)
 		msg.set_message(u8(text));
 
-	{
+	tm.qsText = text;
+
+	{ // Happy easter
 		char m[29] = {0117, 0160, 0145, 0156, 040, 0164, 0150, 0145, 040, 0160, 0157, 0144, 040, 0142, 0141, 0171, 040, 0144, 0157, 0157, 0162, 0163, 054, 040, 0110, 0101, 0114, 056, 0};
-		if (msg.message() == m) {
+		if (msg.channel_id_size() == 1 && msg.channel_id(0) == 0 && msg.message() == m) {
 			PERM_DENIED_TYPE(H9K);
 			return;
 		}
@@ -996,13 +1102,15 @@ void Server::msgTextMessage(ServerUser *uSource, MumbleProto::TextMessage &msg)
 		if (! c)
 			return;
 
-		if (! ChanACL::hasPermission(uSource, c, ChanACL::TextMessage, acCache)) {
+		if (! ChanACL::hasPermission(uSource, c, ChanACL::TextMessage, &acCache)) {
 			PERM_DENIED(uSource, c, ChanACL::TextMessage);
 			return;
 		}
 
 		foreach(User *p, c->qlUsers)
 			users.insert(static_cast<ServerUser *>(p));
+
+		tm.qlChannels.append(id);
 	}
 
 	for (int i=0;i<msg.tree_id_size(); ++i) {
@@ -1012,17 +1120,19 @@ void Server::msgTextMessage(ServerUser *uSource, MumbleProto::TextMessage &msg)
 		if (! c)
 			return;
 
-		if (! ChanACL::hasPermission(uSource, c, ChanACL::TextMessage, acCache)) {
+		if (! ChanACL::hasPermission(uSource, c, ChanACL::TextMessage, &acCache)) {
 			PERM_DENIED(uSource, c, ChanACL::TextMessage);
 			return;
 		}
 
 		q.enqueue(c);
+
+		tm.qlTrees.append(id);
 	}
 
 	while (! q.isEmpty()) {
 		Channel *c = q.dequeue();
-		if (ChanACL::hasPermission(uSource, c, ChanACL::TextMessage, acCache)) {
+		if (ChanACL::hasPermission(uSource, c, ChanACL::TextMessage, &acCache)) {
 			foreach(Channel *sub, c->qlChannels)
 				q.enqueue(sub);
 			foreach(User *p, c->qlUsers)
@@ -1033,17 +1143,23 @@ void Server::msgTextMessage(ServerUser *uSource, MumbleProto::TextMessage &msg)
 	for (int i=0;i < msg.session_size(); ++i) {
 		unsigned int session = msg.session(i);
 		ServerUser *u = qhUsers.value(session);
-		if (! ChanACL::hasPermission(uSource, u->cChannel, ChanACL::TextMessage, acCache)) {
-			PERM_DENIED(uSource, u->cChannel, ChanACL::TextMessage);
-			return;
+		if (u) {
+			if (! ChanACL::hasPermission(uSource, u->cChannel, ChanACL::TextMessage, &acCache)) {
+				PERM_DENIED(uSource, u->cChannel, ChanACL::TextMessage);
+				return;
+			}
+			users.insert(u);
 		}
-		users.insert(u);
+
+		tm.qlSessions.append(session);
 	}
 
 	users.remove(uSource);
 
 	foreach(ServerUser *u, users)
 		sendMessage(u, msg);
+
+	emit userTextMessage(uSource, tm);
 }
 
 void Server::msgACL(ServerUser *uSource, MumbleProto::ACL &msg) {
@@ -1164,14 +1280,19 @@ void Server::msgACL(ServerUser *uSource, MumbleProto::ACL &msg) {
 			g->bInherit = group.inherit();
 			g->bInheritable = group.inheritable();
 			for (int j=0;j<group.add_size();++j)
-				g->qsAdd << group.add(j);
+				if (!getUserName(group.add(j)).isEmpty())
+					g->qsAdd << group.add(j);
 			for (int j=0;j<group.remove_size();++j)
-				g->qsRemove << group.remove(j);
+				if (!getUserName(group.remove(j)).isEmpty())
+					g->qsRemove << group.remove(j);
 			g->qsTemporary = hOldTemp.value(g->qsName);
 		}
 
 		for (int i=0;i<msg.acls_size(); ++i) {
 			const MumbleProto::ACL_ChanACL &mpacl = msg.acls(i);
+			if (mpacl.has_user_id() && getUserName(mpacl.user_id()).isEmpty())
+				continue;
+
 			a = new ChanACL(c);
 			a->bApplyHere=mpacl.apply_here();
 			a->bApplySubs=mpacl.apply_subs();
@@ -1277,7 +1398,7 @@ void Server::msgCryptSetup(ServerUser *uSource, MumbleProto::CryptSetup &msg) {
 	}
 }
 
-void Server::msgContextActionAdd(ServerUser *, MumbleProto::ContextActionAdd &) {
+void Server::msgContextActionModify(ServerUser *, MumbleProto::ContextActionModify &) {
 }
 
 void Server::msgContextAction(ServerUser *uSource, MumbleProto::ContextAction &msg) {
@@ -1303,7 +1424,8 @@ void Server::msgVersion(ServerUser *uSource, MumbleProto::Version &msg) {
 		if (msg.has_os_version())
 			uSource->qsOSVersion = u8(msg.os_version());
 	}
-	log(uSource, QString("Client version %1.%2.%3 (%4: %5)").arg(uSource->uiVersion >> 16).arg((uSource->uiVersion >> 8) & 0xff).arg(uSource->uiVersion & 0xFF).arg(uSource->qsOS).arg(uSource->qsRelease));
+
+	log(uSource, QString("Client version %1 (%2: %3)").arg(MumbleVersion::toString(uSource->uiVersion)).arg(uSource->qsOS).arg(uSource->qsRelease));
 }
 
 void Server::msgUserList(ServerUser *uSource, MumbleProto::UserList &msg) {
@@ -1421,8 +1543,6 @@ void Server::msgUserStats(ServerUser*uSource, MumbleProto::UserStats &msg) {
 	const CryptState &cs = pDstServerUser->csCrypt;
 	const BandwidthRecord &bwr = pDstServerUser->bwr;
 	const QList<QSslCertificate> &certs = pDstServerUser->peerCertificateChain();
-	MumbleProto::UserStats_Stats *mpusss;
-	MumbleProto::Version *mpv;
 
 	bool extend = (uSource == pDstServerUser) || hasPermission(uSource, qhChannels.value(0), ChanACL::Register);
 
@@ -1449,6 +1569,8 @@ void Server::msgUserStats(ServerUser*uSource, MumbleProto::UserStats &msg) {
 	}
 
 	if (local) {
+		MumbleProto::UserStats_Stats *mpusss;
+
 		mpusss = msg.mutable_from_client();
 		mpusss->set_good(cs.uiGood);
 		mpusss->set_late(cs.uiLate);
@@ -1470,6 +1592,8 @@ void Server::msgUserStats(ServerUser*uSource, MumbleProto::UserStats &msg) {
 	msg.set_tcp_ping_var(pDstServerUser->dTCPPingVar);
 
 	if (details) {
+		MumbleProto::Version *mpv;
+
 		mpv = msg.mutable_version();
 		if (pDstServerUser->uiVersion)
 			mpv->set_version(pDstServerUser->uiVersion);
@@ -1483,6 +1607,7 @@ void Server::msgUserStats(ServerUser*uSource, MumbleProto::UserStats &msg) {
 
 		foreach(int v, pDstServerUser->qlCodecs)
 			msg.add_celt_versions(v);
+		msg.set_opus(pDstServerUser->bOpus);
 
 		msg.set_address(pDstServerUser->haAddress.toStdString());
 	}
@@ -1497,6 +1622,8 @@ void Server::msgUserStats(ServerUser*uSource, MumbleProto::UserStats &msg) {
 }
 
 void Server::msgRequestBlob(ServerUser *uSource, MumbleProto::RequestBlob &msg) {
+	MSG_SETUP_NO_UNIDLE(ServerUser::Authenticated);
+
 	int ntextures = msg.session_texture_size();
 	int ncomments = msg.session_comment_size();
 	int ndescriptions = msg.channel_description_size();
@@ -1540,3 +1667,6 @@ void Server::msgRequestBlob(ServerUser *uSource, MumbleProto::RequestBlob &msg)
 
 void Server::msgServerConfig(ServerUser *, MumbleProto::ServerConfig &) {
 }
+
+void Server::msgSuggestConfig(ServerUser *, MumbleProto::SuggestConfig &) {
+}
