@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -28,15 +28,27 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifndef _META_H
-#define _META_H
+#ifndef META_H_
+#define META_H_
+
+#include <QtCore/QDir>
+#include <QtCore/QList>
+#include <QtCore/QUrl>
+#include <QtCore/QVariant>
+#include <QtNetwork/QHostAddress>
+#include <QtNetwork/QSslCertificate>
+#include <QtNetwork/QSslKey>
+#ifdef Q_OS_WIN
+#include <windows.h>
+#endif
 
-#include "murmur_pch.h"
 #include "Timer.h"
-#include "Server.h"
-#include "DBus.h"
 
-struct MetaParams {
+class Server;
+class QSettings;
+
+class MetaParams {
+public:
 	QDir qdBasePath;
 
 	QList<QHostAddress> qlBind;
@@ -46,8 +58,11 @@ struct MetaParams {
 	int iMaxUsers;
 	int iMaxUsersPerChannel;
 	int iDefaultChan;
+	bool bRememberChan;
 	int iMaxTextMessageLength;
 	int iMaxImageMessageLength;
+	int iOpusThreshold;
+	int iChannelNestingLimit;
 	bool bAllowHTML;
 	QString qsPassword;
 	QString qsWelcomeText;
@@ -77,11 +92,12 @@ struct MetaParams {
 	QString qsLogfile;
 	QString qsPid;
 	QString qsIceEndpoint;
-	QString qsIceSecret;
+	QString qsIceSecretRead, qsIceSecretWrite;
 
 	QString qsRegName;
 	QString qsRegPassword;
 	QString qsRegHost;
+	QString qsRegLocation;
 	QUrl qurlRegWeb;
 	bool bBonjour;
 
@@ -94,13 +110,25 @@ struct MetaParams {
 
 	QMap<QString, QString> qmConfig;
 
+#ifdef Q_OS_UNIX
 	unsigned int uiUid, uiGid;
+	QString qsHome;
+	QString qsName;
+#endif
+
+	QVariant qvSuggestVersion;
+	QVariant qvSuggestPositional;
+	QVariant qvSuggestPushToTalk;
 
 	QSettings *qsSettings;
 
 	MetaParams();
 	~MetaParams();
 	void read(QString fname = QString("murmur.ini"));
+
+private:
+		template <class T>
+		T typeCheckedFromSettings(const QString &name, const T &variable);
 };
 
 class Meta : public QObject {
@@ -136,6 +164,4 @@ class Meta : public QObject {
 
 extern Meta *meta;
 
-#else
-class Meta;
 #endif
